@@ -859,6 +859,11 @@ public:
     return LP64_ONLY(true) NOT_LP64(false); // not implemented on x86_32
   }
 
+  // x86_64 supports secondary supers table
+  constexpr static bool supports_secondary_supers_table() {
+    return LP64_ONLY(true) NOT_LP64(false); // not implemented on x86_32
+  }
+
   constexpr static bool supports_stack_watermark_barrier() {
     return true;
   }
